@@ -7,10 +7,12 @@ import sys
 from argparse import ArgumentParser, RawDescriptionHelpFormatter
 
 
-__version__ = '0.1'
+__version__ = '0.2'
 __author__ = 'Hideo Hattroi <hhatto.jp@gmail.com>'
 __license__ = 'NewBSDLicense'
 
+__all__ = ["main", "ZshCompletionGenerator", "HelpParser"]
+
 
 class InvalidParserTypeError(Exception):
     """Base Class for invalid parser type exception."""
@@ -28,21 +30,23 @@ def get_parser_type(parser_obj):
     return parser_type
 
 
-def _escape_squarebracket(strings):
-    """escape to only squarebracket.
+def _escape_strings(strings):
+    """escape to squarebracket and doublequote.
 
-    >>> print _escape_squarebracket("hoge")
+    >>> print _escape_strings("hoge")
     hoge
-    >>> print _escape_squarebracket("[hoge")
+    >>> print _escape_strings("[hoge")
     \[hoge
-    >>> print _escape_squarebracket("hoge]")
+    >>> print _escape_strings("hoge]")
     hoge\]
-    >>> print _escape_squarebracket("[hoge]")
+    >>> print _escape_strings("[hoge]")
     \[hoge\]
+    >>> print _escape_strings('[ho"ge]')
+    \[ho\"ge\]
     """
     ret = []
     for string in strings:
-        if string == '[' or string == ']':
+        if string == '[' or string == ']' or string == '"':
             string = '\\' + string
         ret.append(string)
     return "".join(ret)
@@ -52,10 +56,12 @@ class ZshCompletionGenerator(object):
 
     """Generator of Zsh Completion Function"""
 
-    def __init__(self, commandname=None, parser=None):
+    def __init__(self, commandname=None, parser=None, parser_type=None):
         self.commandname = commandname
         self.parser = parser
-        self.parser_type = get_parser_type(parser)
+        if not parser_type:
+            parser_type = get_parser_type(parser)
+        self.parser_type = parser_type
 
     def _get_dircomp(self, opt):
         """judged to directories and files completion.
@@ -109,9 +115,12 @@ class ZshCompletionGenerator(object):
                 opts = action.option_strings
             for opt in opts:
                 directory_comp = self._get_dircomp(opt)
-                tmp = "  \"%s[%s]%s%s\" \\" % (opt,
-                      _escape_squarebracket(action.help),
-                      metavar, directory_comp)
+                if action.help:
+                    tmp = "  \"%s[%s]%s%s\" \\" % (opt,
+                          _escape_strings(action.help),
+                          metavar, directory_comp)
+                else:
+                    tmp = "  \"%s%s%s\" \\" % (opt, metavar, directory_comp)
                 ret.append(tmp)
         ret.append("  \"*::args:_files\"")
         return "\n".join(ret)
@@ -204,32 +213,42 @@ class HelpParser(object):
         option_list = []
         ## 1 is 'Options' line
         for line in self.parselines[1:]:
-            if len(line) < helpstring_offset:
-                break
-            if '--help' in line:
+            if line.isspace() or not len(line) or '--help     ' in line:
+                ## FIXME: judged to '--help     ' is durty hack...
                 continue
             tmp = line.split()
             metavar = None
-            if tmp[0][:2] == '--':
+            if line.find('--') < helpstring_offset and tmp[0][:2] == '--':
                 ## only long option
                 longopt = tmp[0]
+                longopt_length = 0
                 if '=' in longopt:
                     longtmp = longopt.split("=")
                     longopt = longtmp[0]
                     metavar = longtmp[1]
+                    longopt_length += len(metavar) + 1
+                longopt_length += len(longopt) + line.find('--')
+                ## check to exist help strings
+                if longopt_length > helpstring_offset:
+                    helpstrings = ""
+                else:
+                    helpstrings = line[helpstring_offset:] + ' '
                 option_list.append({'short': None,
                                     'long': longopt,
                                     'metavar': metavar,
-                                    'help': line[helpstring_offset:] + ' '})
+                                    'help': helpstrings})
                 option_cnt += 1
-            elif tmp[0][0] == '-':
+            elif line.find('--') < helpstring_offset and tmp[0][0] == '-':
                 ## short option
-                shortopt = tmp[0][:2]
+                shortopt = line.split(', ')[0].lstrip().split()[0]
                 longopt = None
-                if line[helpstring_offset - 1] is ' ':
+                if len(line) < helpstring_offset:
+                    help_string = ""
+                elif line[helpstring_offset - 1] is ' ':
                     help_string = line[helpstring_offset:]
                 else:
                     help_string = ""
+                tmp = line.split(', ')
                 if tmp[1][:2] == '--':
                     longopt = tmp[1]
                     if '=' in longopt:
@@ -253,24 +272,29 @@ class HelpParser(object):
 
 def main():
     """tool main"""
-    from select import poll, POLLIN
+    from select import poll, POLLIN, POLLHUP
     oparser = OptionParser(version=__version__,
                            description=__doc__,
                            usage="usage: genzshcomp FILE\n"\
                                  "             or\n"\
                                  "       USER_SCRIPT --help | genzshcomp")
     (opts, args) = oparser.parse_args()
-    if len(args):
+    if sys.stdin.isatty() and len(args):
         helptext = open(args[0]).read()
+    elif sys.stdin.isatty() and not len(args):
+        oparser.print_help()
+        return -1
     else:
-        poller = poll()
-        poller.register(sys.stdin, POLLIN)
-        ret = poller.poll(300)
-        if len(ret) == 1 and ret[0][1] & POLLIN:
-            helptext = sys.stdin.read()
-        else:
-            oparser.print_help()
-            return -1
+        helptext = ""
+        while True:
+            poller = poll()
+            poller.register(sys.stdin, POLLIN)
+            ret = poller.poll()
+            if len(ret) == 1 and ret[0][1] & POLLIN:
+                helptext += sys.stdin.read()
+            elif len(ret) == 1 and ret[0][1] & POLLHUP:
+                helptext += sys.stdin.read()
+                break
     help_parser = HelpParser(helptext)
     command_name = help_parser.get_commandname()
     option_parser = help_parser.help2optparse()
